@@ -16,7 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * v4l input plugin
  */
@@ -882,7 +882,7 @@ static int open_video_capture_device(v4l_input_plugin_t *this)
       if (ret < 0) {
 	close (this->video_fd);
 	this->video_fd = -1;
-	lprintf("Grab: no colorspace format found\n");
+	lprintf("Grab: no colour space format found\n");
 	return 0;
       }
       else
