@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with self program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: audio_out.c,v 1.210 2007/04/01 00:52:36 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * 22-8-2001 James imported some useful AC3 sections from the previous alsa driver.
  *   (c) 2001 Andy Lo A Foe <andy@alsaplayer.org>
@@ -53,7 +51,6 @@
  *   Copyright (C) 2002  Felipe Rivera <liebremx at users sourceforge net>
  *
  * heavily modified by guenter bartsch 2003 for use in libxine
- *
  */
 
 #ifndef	__sun
@@ -290,6 +287,7 @@ struct audio_fifo_s {
   int                num_buffers;
 };
 
+static int ao_set_property (xine_audio_port_t *this_gen, int property, int value);
 
 static audio_fifo_t *fifo_new (xine_t *xine) {
 
@@ -340,9 +338,7 @@ static void fifo_append (audio_fifo_t *fifo,
   pthread_mutex_unlock (&fifo->mutex);
 }
 
-static audio_buffer_t *fifo_remove_int (audio_fifo_t *fifo, int blocking) {
-  audio_buffer_t *buf;
-
+static audio_buffer_t *fifo_peek_int (audio_fifo_t *fifo, int blocking) {
   while (!fifo->first) {
     pthread_cond_signal (&fifo->empty);
     if (blocking)
@@ -357,25 +353,38 @@ static audio_buffer_t *fifo_remove_int (audio_fifo_t *fifo, int blocking) {
         return NULL;
     }
   }
+  return fifo->first;
+}
 
-  buf = fifo->first;
+static audio_buffer_t *fifo_remove_int (audio_fifo_t *fifo, int blocking) {
+  audio_buffer_t *buf = fifo_peek_int(fifo, blocking);
+  if (!buf)
+    return NULL;
 
-  if (buf) {
-    fifo->first = buf->next;
+  fifo->first = buf->next;
 
-    if (!fifo->first) {
+  if (!fifo->first) {
 
-      fifo->last = NULL;
-      fifo->num_buffers = 0;
-      pthread_cond_signal (&fifo->empty);
+    fifo->last = NULL;
+    fifo->num_buffers = 0;
+    pthread_cond_signal (&fifo->empty);
 
-    } else 
-      fifo->num_buffers--;
-
-  }
+  } else
+    fifo->num_buffers--;
 
   buf->next = NULL;
-    
+
+  return buf;
+}
+
+static audio_buffer_t *fifo_peek (audio_fifo_t *fifo) {
+
+  audio_buffer_t *buf;
+
+  pthread_mutex_lock (&fifo->mutex);
+  buf = fifo_peek_int(fifo, 1);
+  pthread_mutex_unlock (&fifo->mutex);
+
   return buf;
 }
 
@@ -1000,7 +1009,7 @@ static void *ao_loop (void *this_gen) {
     
     if (!in_buf) {
       lprintf ("loop: get buf from fifo\n");
-      in_buf = fifo_remove (this->out_fifo);
+      in_buf = fifo_peek (this->out_fifo);
       bufs_since_sync++;
       lprintf ("got a buffer\n");
     }
@@ -1013,6 +1022,7 @@ static void *ao_loop (void *this_gen) {
     }
 
     if (this->discard_buffers) {
+      fifo_remove (this->out_fifo);
       if (in_buf->stream)
 	_x_refcounter_dec(in_buf->stream->refcounter);
       fifo_append (this->free_fifo, in_buf);
@@ -1039,6 +1049,7 @@ static void *ao_loop (void *this_gen) {
 	cur_time = this->clock->get_current_time (this->clock);
 	if (in_buf->vpts < cur_time ) {
 	  lprintf ("loop: next fifo\n");
+	  fifo_remove (this->out_fifo);
 	  if (in_buf->stream)
 	    _x_refcounter_dec(in_buf->stream->refcounter);
 	  fifo_append (this->free_fifo, in_buf);
@@ -1163,6 +1174,7 @@ static void *ao_loop (void *this_gen) {
 
       /* drop package */
       lprintf ("loop: drop package, next fifo\n");
+      fifo_remove (this->out_fifo);
       if (in_buf->stream)
 	_x_refcounter_dec(in_buf->stream->refcounter);
       fifo_append (this->free_fifo, in_buf);
@@ -1229,6 +1241,7 @@ static void *ao_loop (void *this_gen) {
       } else {
         result = 0;
       }
+      fifo_remove (this->out_fifo);
       
       if( result < 0 ) {
         /* device unplugged. */
@@ -1599,6 +1612,11 @@ static void ao_close(xine_audio_port_t *this_gen, xine_stream_t *stream) {
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_out: no streams left, closing driver\n");
 
     if (this->audio_loop_running) {
+      if (this->clock->speed == XINE_SPEED_PAUSE ||
+          (this->clock->speed != XINE_FINE_SPEED_NORMAL && !this->slow_fast_audio)) {
+        /* discard buffers, otherwise we'll wait forever */
+        ao_set_property(this_gen, AO_PROP_DISCARD_BUFFERS, 1);
+      }
       /* make sure there are no more buffers on queue */
       fifo_wait_empty(this->out_fifo);
     }
