@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xine.c,v 1.342 2007/02/20 00:37:02 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
@@ -37,7 +35,8 @@
 #include <pthread.h>
 #include <stdarg.h>
 #include <stdio.h>
-#if defined (__linux__)
+#include <ctype.h>
+#if defined (__linux__) || defined (__GLIBC__)
 #include <endian.h>
 #elif defined (__FreeBSD__)
 #include <machine/endian.h>
@@ -781,9 +780,9 @@ void _x_flush_events_queues (xine_stream_t *stream) {
   pthread_mutex_unlock (&stream->event_queues_lock);
 }
 
-static int open_internal (xine_stream_t *stream, const char *mrl) {
+/*static*/ int open_internal (xine_stream_t *stream, const char *mrl) {
 
-  const char *stream_setup;
+  const char *stream_setup = NULL;
   int no_cache = 0;
   
   if (!mrl) {
@@ -808,13 +807,19 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
    * look for a stream_setup in MRL and try finding an input plugin
    */
 
-  stream_setup = mrl;
-  /* look for the next '#' or try the whole MRL, if none is found */
-  while (*stream_setup &&
-	(stream_setup = (strchr(stream_setup, '#') ? strchr(stream_setup, '#') : strlen(mrl) + mrl))) {
-    char *input_source = (char *)malloc(stream_setup - mrl + 1);
-    memcpy(input_source, mrl, stream_setup - mrl);
-    input_source[stream_setup - mrl] = '\0';
+  if (isalpha (*mrl))
+  {
+    stream_setup = mrl + 1;
+    while (isalnum (*stream_setup) || *stream_setup == '+' || *stream_setup == '-' || *stream_setup == '.')
+      ++stream_setup;
+    if (stream_setup[0] == ':' && stream_setup[1] == '/')
+      stream_setup = strchr (mrl, '#');
+    else
+      stream_setup = NULL;
+  }
+  
+  {
+    char *input_source = strndup (mrl, stream_setup ? stream_setup - mrl : strlen (mrl));
 
     /*
      * find an input plugin
@@ -833,7 +838,6 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
       res = (stream->input_plugin->open) (stream->input_plugin);
       switch(res) {
       case 1: /* Open successfull */
-	free(input_source);
 	break;
       case -1: /* Open unsuccessfull, but correct plugin */
 	free(input_source);
@@ -846,14 +850,9 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	stream->input_plugin = NULL;
 	stream->err = XINE_ERROR_INPUT_FAILED;
       }
-      if ( res ) break;
     }
 
     free(input_source);
-    /* if we fail when passing up to the first '#' to the input plugins,
-     * maybe the user stated a (invalid) MRL, with a '#' belonging to the
-     * input source -> look for the next '#' and try again */
-    if (*stream_setup) stream_setup++;
   }
   
   if (!stream->input_plugin) {
@@ -863,7 +862,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
     return 0;
   }
 
-  if (*stream_setup) {
+  if (stream_setup) {
 
     while (stream_setup && *stream_setup && *(++stream_setup)) {
       if (strncasecmp(stream_setup, "demux", 5) == 0) {
@@ -1195,7 +1194,11 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
   stream->demux_plugin->send_headers (stream->demux_plugin);
 
   if (stream->demux_plugin->get_status(stream->demux_plugin) != DEMUX_OK) {
-    xine_log (stream->xine, XINE_LOG_MSG, _("xine: demuxer failed to start\n"));
+    if (stream->demux_plugin->get_status(stream->demux_plugin) == DEMUX_FINISHED) {
+      xine_log (stream->xine, XINE_LOG_MSG, _("xine: demuxer is already done. that was fast!\n"));
+    } else {
+      xine_log (stream->xine, XINE_LOG_MSG, _("xine: demuxer failed to start\n"));
+    }
 
     _x_free_demux_plugin(stream, stream->demux_plugin);
     stream->demux_plugin = NULL;
@@ -2055,6 +2058,9 @@ const char *const *xine_get_log_names (xine_t *this) {
 
 static inline void check_log_alloc (xine_t *this, int buf)
 {
+  if ( this->log_buffers[buf] )
+    return;
+
   pthread_mutex_lock (&this->log_lock);
 
   if ( ! this->log_buffers[buf] )
